%% MAKE_QUAKE_XY
%
% Use this file to simulate an earthquake on the X-Y shake table system 
% when the recorded data is in gravitational acceleration units (g). 
% The q_scale algorithm is used to compute the desired position such that 
% the measured accelerations yielded on the shake table are equivalent to 
% the recorded values.
%
% Copyright (C) 2015 Quanser Consulting Inc.
% Quanser Consulting Inc.
%
%% INPUT
% name of data source file:
% Northridge
input_filename_x = 'RSN1086_NORTHR_SYL090.AT2';
input_filename_y = 'RSN1086_NORTHR_SYL360.AT2';
input_filename_z = 'RSN1086_NORTHR_SYL-UP.AT2';
% Kobe
% input_filename_x = 'RSN1105_KOBE_HIK000.AT2';
% input_filename_y = 'RSN1105_KOBE_HIK090.AT2';
% input_filename_z = 'RSN1105_KOBE_HIK-UP.AT2';
% Cape Mendocino
% input_filename_x = 'RSN825_CAPEMEND_CPM000.AT2';
% input_filename_y = 'RSN825_CAPEMEND_CPM090.AT2';
% input_filename_z = 'RSN825_CAPEMEND_CPM-UP.AT2';
% El Centro
% input_filename_x = 'RSN180_IMPVALL.H_H-E05140.AT2';
% input_filename_y = 'RSN180_IMPVALL.H_H-E05230.AT2';
% input_filename_z = 'RSN180_IMPVALL.H_H-E05-UP.AT2';
%
% Maximum scaled position (cm). NOTE: Do not exceed stroke limit. It is
% recommend you keep this value below 5.0 cm for most trajectories.
x_max = 2.5;
%
%% MAKE QUAKE
% load recorded acceleration earthquake data in workspace (g) and sample
% time (s)
[dt_x, acc_data_x] = init_earthquake_data(input_filename_x);
[dt_y, acc_data_y] = init_earthquake_data(input_filename_y);
[dt_z, acc_data_z] = init_earthquake_data(input_filename_z);
% construct trajectory: 
% tq = recorded earthquake time (s)
% a = record earthquake acceleration (g)
[tqx, ax] = construct_quake_trajectory(acc_data_x,dt_x);
[tqy, ay] = construct_quake_trajectory(acc_data_y,dt_y);
[tqz, az] = construct_quake_trajectory(acc_data_z,dt_z);
%
% Compute desired position of STII to achieve actual quake accelerations.
%   Tc =  Scaled time array (s)
%   Xc =  Scaled position array - use for table position command (cm)
%   Ac =  Scaled acceleration array using d2 Xc / dt2 (g)
%   Tu =  Earthquake time array (s)
%   Xu =  Computed earthquake displacement (not scaled) (cm)
%   Au =  Earthquake acceleration array (g), scaled to Tu (g)
[Tc_x, Xc_x, Ac_x, Tu_x, Xu_x, Au_x] = q_scale(tqx, ax, x_max);
[Tc_y, Xc_y, Ac_y, Tu_y, Xu_y, Au_y] = q_scale(tqy, ay, x_max);
[Tc_z, Xc_z, Ac_z, Tu_z, Xu_z, Au_z] = q_scale(tqz, az, x_max);
% 
%% HEXAPOD Commands
% Convert desired scaled position and acceleration for Hexapod m)
Xd_x = 10*Xc_x; % cm to mm
Xd_y = 10*Xc_y; % cm to mm
Xd_z = 10*Xc_z; % cm to mm
Ad_x = 9.81*Ac_x; % acceleration in 'm/s^2'
Ad_y = 9.81*Ac_y; % acceleration in 'm/s^2'
Ad_z = 9.81*Ac_z; % acceleration in 'm/s^2'
% final time (s)
tf = max([max(Tc_x), max(Tc_y), max(Tc_z)]);
%
%% Plot X-Axis
figure(3);
set(3,'name','Q-Scale X-Axis');
% Original recorded earthquake acceleration
subplot(2,2,1);
plot(Tu_x,Au_x);
title('Earthquake Acceleration (g)');
% axis([0 max(Tu_x) 1.5*min(Au_x) 1.5*max(Au_x)]);
%
% Table acceleration from scaling (same as recorded)
subplot(2,2,2)
plot(Tc_x,Ac_x,'r-');
title('Table Acceleration (g)');
% axis([0 max(Tc_x)  1.5*min(Ac_x) 1.5*max(Ac_x)]);
% 
% Original recorded earthquake position
subplot(2,2,3);
plot(Tu_x,Xu_x,'r-');
xlabel('time (s)');
title('Earthquake Position (cm)');
% axis([0 max(Tu_x) 1.5*min(Xu_x) 1.5*max(Xu_x)]);
% 
% Scaled/command table position
subplot(2,2,4);
plot(Tc_x,Xc_x,'r-');
xlabel('time (s)');
title('Table Position (cm)');
% axis([0 max(Tc_x) 1.5*min(Xc_x) 1.5*max(Xc_x)]);
% 
%% Plot Y-Axis
figure(4)
set(4,'name','Q-Scale Y-Axis');
% Original recorded earthquake acceleration
subplot(2,2,1);
plot(Tu_y,Au_y);
title('Earthquake Acceleration (g)');
% axis([0 max(Tu_y) 1.5*min(Au_y) 1.5*max(Au_y)]);
%
% Table acceleration from scaling (same as recorded)
subplot(2,2,2)
plot(Tc_y,Ac_y,'r-');
title('Table Acceleration (g)');
% axis([0 max(Tc_y)  1.5*min(Ac_y) 1.5*max(Ac_y)]);
% 
% Original recorded earthquake position
subplot(2,2,3);
plot(Tu_y,Xu_y,'r-');
xlabel('time (s)');
title('Earthquake Position (cm)');
% axis([0 max(Tu_y) 1.5*min(Xu_y) 1.5*max(Xu_y)]);
% 
% Scaled/command table position
subplot(2,2,4);
plot(Tc_y,Xc_y,'r-');
xlabel('time (s)');
title('Table Position (cm)');
% axis([0 max(Tc_y) 1.5*min(Xc_y) 1.5*max(Xc_y)]);
% 
%% Plot Z-Axis
figure(5)
set(5,'name','Q-Scale Z-Axis');
% Original recorded earthquake acceleration
subplot(2,2,1);
plot(Tu_z,Au_z);
title('Earthquake Acceleration (g)');
% axis([0 max(Tu_z) 1.5*min(Au_z) 1.5*max(Au_z)]);
%
% Table acceleration from scaling (same as recorded)
subplot(2,2,2)
plot(Tc_z,Ac_z,'r-');
title('Table Acceleration (g)');
% axis([0 max(Tc_z)  1.5*min(Ac_z) 1.5*max(Ac_z)]);
% 
% Original recorded earthquake position
subplot(2,2,3);
plot(Tu_z,Xu_z,'r-');
xlabel('time (s)');
title('Earthquake Position (cm)');
% axis([0 max(Tu_z) 1.5*min(Xu_z) 1.5*max(Xu_z)]);
% 
% Scaled/command table position
subplot(2,2,4);
plot(Tc_z,Xc_z,'r-');
xlabel('time (s)');
title('Table Position (cm)');
% axis([0 max(Tc_z) 1.5*min(Xc_z) 1.5*max(Xc_z)]);
%
%
% Clear un-used variables
clear [Tu_x, Xu_x, Au_x];
clear [Tu_y, Xu_y, Au_y];
clear [Tu_z, Xu_z, Au_z];
% 
% Save scaling results
% print (3,'-dpng','-r300','scaling_x.png');
% print (4,'-dpng','-r300','scaling_y.png');
% print (5,'-dpng','-r300','scaling_z.png');